#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <alloca.h>
#include "main.h"
#include "misc.h"
#include "wrapper.h"

extern struct fd_ll *fds;
extern struct metar_data_ll *metardata;
int in_html = 0, in_char = 0;

int readln(int fd, char *buf)
{
	char *mybuf, *ptr;
	struct fd_ll *f;
	int result;

	mybuf = alloca(BUF_SIZE);

	if ((result = recv(fd, mybuf, BUF_SIZE, MSG_PEEK)) < 0)
		return -1;
	mybuf[result] = 0;

	if (!result)
		return -2;

	if (!(f = find_fd(fd))) {
		return -1;
	}

	if (!(ptr = strchr(mybuf, '\n'))) {
		recv(fd, mybuf, result, 0);
		if (f->buf) {
			f->buf = my_realloc(f->buf, strlen(f->buf) + strlen(mybuf));
			strcpy(f->buf + strlen(f->buf), mybuf);
		} else {
			f->buf = my_malloc(result);
			strcpy(f->buf, mybuf);
		}
		return -1;
	}
	*ptr = 0;

	if (f->buf) {
		strcpy(buf, f->buf);
		recv(fd, buf + strlen(f->buf), strlen(mybuf) + 1, 0);
		buf[strlen(mybuf) + strlen(f->buf)] = 0;

		free(f->buf);
		f->buf = NULL;
	} else {
		recv(fd, buf, strlen(mybuf) + 1, 0);
		buf[strlen(mybuf)] = 0;
	}

	strip_newline(buf);

	return strlen(buf);
}

void add_fd(int fd, char *id, void (*handler)())
{
	struct fd_ll *f = fds, *new;

	while (f) {
		if (f->fd == fd)
			return;
		f = f->next;
	}
	new = my_malloc(sizeof(struct fd_ll));

	new->fd = fd;
	new->id = my_strdup(id);
	new->parse_state = 0;
	new->next = NULL;
	new->prev = NULL;
	new->buf = NULL;
	new->handle = handler;

	if (fds) {
		new->next = fds;
		fds->prev = new;
	}
	fds = new;
}

void del_fd(int fd)
{
	struct fd_ll *f = fds;

	while (f) {
		if (f->fd == fd)
			break;
		f = f->next;
	}
	if (!f)
		return;
	close(f->fd);
	free(f->id);

	if (f->prev) {
		if ((f->prev->next = f->next))
			f->next->prev = f->prev;
	} else {
		if ((fds = f->next))
			fds->prev = NULL;
	}

	free(f);
}

void del_all_fd(void)
{
	struct fd_ll *f = fds;
	printf("deleting\n");

	while (f) {
		del_fd(f->fd);
		f = fds;
	}
}

struct fd_ll *find_fd(int fd)
{
	struct fd_ll *f = fds;

	while (f) {
		if (f->fd == fd)
			return f;
		f = f->next;
	}
	return (struct fd_ll *)NULL;
}


void split(char *dest, char *source, int delim)
{
	char *ptr;

	ptr = strchr(source, delim);

	if (!ptr) {
		if (dest)
			strcpy(dest, source);

		*source = 0;
		return;
	}
	*ptr = 0;

	if (dest)
		strcpy(dest, source);

	strcpy(source, ptr + 1);
}

/* idea for this was taken from hop's function lookup code in ircii-EPIC. thanks! */
/* okay, more like ripped and then scaled down for my needs! ;) */
int is_array_item(char *array[], size_t size, char *item)
{
	int min = 0, max = size, pos, prev = -1, match;
	
	for (;;) {
		pos = (max + min) / 2;
		if (pos == prev)
			return 0;
		prev = pos;
		
		match = strcasecmp(item, array[pos]);
/*		printf("[pos:%d min:%d max:%d / %d = strcasecmp(%s, %s);]\n", pos, min, max, match, array[pos], item); */
		
		if (match == 0)
			return 1;
		if (match > 0)
			min = pos;
		else
			max = pos;
	}
}

char **separate_data(char *string)
{
	int count, x;
	char **code, *buf, *ptr;

	for (count = 0, ptr = string; (ptr = strchr(ptr, ' ')); count++, ptr++);
	count++;

	code = my_malloc((count + 1) * sizeof(char *));

	buf = my_malloc(strlen(string) + 1);

	for (x = 0; x < count; x++) {
		split(buf, string, ' ');
		code[x] = my_strdup(buf);
	}
	code[x] = NULL;

	free(buf);

	return code;
}

struct metar_data_ll *find_metar_data(char *id)
{
	struct metar_data_ll *m = metardata;
	
	while (m) {
		if (match(m->id, id))
			return m;
		m = m->next;
	}
	return NULL;
}

void del_metar_data(char *id)
{
	struct metar_data_ll *m = find_metar_data(id);

	if (!m)
		return;

	free(m->id);

	if (m->prev) {
		if ((m->prev->next = m->next))
			m->next->prev = m->prev;
	} else {
		if ((metardata = m->next))
			metardata->prev = NULL;
	}

	free(m);
}

void del_all_metar_data(void)
{
	struct metar_data_ll *m = metardata;

	while (m) {
		del_metar_data(m->id);
		m = metardata;
	}
}

void strip_newline(char *buf)
{
	if (buf[strlen(buf) - 1] == '\n')
		buf[strlen(buf) - 1] = '\0';
	if (buf[strlen(buf) - 1] == '\r')
		buf[strlen(buf) - 1] = '\0';
}

char *strip_html(char *s)
{
	int i = 0;
	char *new = my_malloc(strlen(s) + 1);

	while (*s) {
		if (*s == '<')
			in_html++;
		if (*s == '&')
			in_char = 1;
		if (!in_html && !in_char)
			new[i++] = *s;
		if (*s == '>')
			in_html--;
		if ((*s == ';') && in_char) {
			new[i++] = ' ';
			in_char = 0;
		}
		s++;
	}
	new[i] = '\0';
	return new;
}
