#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <alloca.h>
#include "retrieve.h"
#include "main.h"
#include "misc.h"
#include "wrapper.h"

extern struct fd_ll *fds;
extern struct metar_data_ll *metardata;
extern int errno, h_errno;

/* stolen and adapted from wget */
int make_connection(int *sock, char *hostname, unsigned short port)
{
	struct sockaddr_in sock_name;
	struct hostent *hptr;

	if (!(hptr = gethostbyname(hostname)))
		return 1;

	/* Copy the address of the host to socket description.  */
	memcpy(&sock_name.sin_addr, hptr->h_addr_list[0], hptr->h_length);

	/* Set port and protocol */
	sock_name.sin_family = AF_INET;
	sock_name.sin_port = htons(port);

	/* Make an internet socket, stream type.  */
	if ((*sock = socket(AF_INET, SOCK_STREAM, 0)) == -1)
		return 2;

	/* Connect the socket to the remote host.  */
	if (connect(*sock, (struct sockaddr *)&sock_name, sizeof(sock_name))) {
		if (errno == ECONNREFUSED)
			return 3;
		else
			return 4;
	}
	return 0;
}

int metar_request(char *hostname, unsigned short port, char *id, char *request)
{
	int socket;
	struct metar_data_ll *new;

	if (make_connection(&socket, hostname, port) != 0)
		return 1;

	if (write(socket, request, strlen(request)) < strlen(request))
		return 2;

	add_fd(socket, id, metar_request_handler);

	if (!(new = find_metar_data(id))) {
		new = my_malloc(sizeof(struct metar_data_ll));
        new->string = NULL;
		new->id = my_strdup(id);
        new->prev = NULL;
        new->next = NULL;
        
		if (metardata) {
			new->next = metardata;
			metardata->prev = new;
		}
		metardata = new;
	}		
	return 0;
}

void metar_request_handler(int socket)
{
	struct metar_data_ll *m;
	struct fd_ll *f;
	char *buffer = alloca(BUF_SIZE), *line;
	int i = 0, result;
	
	f = find_fd(socket);
	if (!f) {
		printf("metar_request_handler: no fd associated with this data!\n");
		return;
	}

	m = find_metar_data(f->id);
	if (!m) {
		printf("metar_request_handler: no data structure available!\n");
		return;
	}

	if (!m->string) {
		m->string = my_malloc(BUF_SIZE);
		memset(m->string, 0, BUF_SIZE);
	}
	
	while ((result = readln(socket, buffer)) > 0) {
		line = strip_html(buffer);
		for (i = 0; my_isspace(line[i]); i++)
			;
		memmove(line, &line[i], strlen(&line[i]) + 1);
		switch (f->parse_state) {
			case 0:
				if (!strcasecmp(line, "The observation is:"))
					f->parse_state++;
				break;
			case 1:
				if (!strncasecmp(line, f->id, 4))
					f->parse_state++;
				else
					break;
			case 2:
				if (!strlen(line)) {
					f->parse_state++;
					return;
				}
			
				/* Resize buffer if we're about to overflow. */
				if ((BUF_SIZE - strlen(m->string)) < (strlen(line) + 1))
					m->string = my_realloc(m->string, strlen(m->string) + BUF_SIZE);
				if (strlen(m->string) > 0)
					strcat(m->string, " ");
				strcat(m->string, line);

				break;
			default:
				printf("metar_request_handler: parse_state is [%d] (?" "?)\n", f->parse_state);
				return;
		}
	} 
	if (result == -2)
		f->parse_state = 3;
/*	printf("returning with parse_state[%d], m->string %s= NULL, strlen(m->string) = %d\n", f->parse_state, m->string ? "!" : "=", strlen(m->string));  */
	return;
}
